@@ -18,16 +18,14 @@
 #   * DELETE /sessions/my                        : ask for global logout (if GlobalLogout plugin is on)
 #
 # - Authentication
-#   * POST /sessions/<type>/<session-id>?auth    : authenticate with a fixed
-#                                                  sessionId
+#   * POST /sessions/<type>/<session-id>?auth    : authenticate with a fixed sessionId
 #   * Note that the "getCookie" method (authentification via SOAP) exists for
 #      REST requests directly by using '/' path : the portal recognize REST
 #      calls and generate JSON response instead of web page.
 #
 # - Configuration (if restConfigServer is on)
 #   * GET /config/latest                          : get the last config metadata
-#   * GET /config/<cfgNum>                        : get the metadata for config
-#                                                  n° <cfgNum>
+#   * GET /config/<cfgNum>                        : get the metadata for config n° <cfgNum>
 #   * GET /config/<latest|cfgNum>/<key>           : get conf key value
 #   * GET /config/<latest|cfgNum>?full            : get the full configuration
 #   where <type> is the session type ("global" for SSO session or "persistent")
@@ -66,7 +64,7 @@ use Lemonldap::NG::Portal::Main::Constants qw(
   PE_PASSWORD_OK
 );
 
-our $VERSION = '2.0.16';
+our $VERSION = '2.16.1';
 
 extends 'Lemonldap::NG::Portal::Main::Plugin';
 
@@ -78,37 +76,36 @@ has configStorage => (
     }
 );
 has exportedAttr => (
-    is      => 'rw',
+    is      => 'ro',
     lazy    => 1,
     default => sub {
+        my %attributes;
         my $conf = $_[0]->{conf};
-        if ( $conf->{exportedAttr} and $conf->{exportedAttr} !~ /^\s*\+/ ) {
-            return '[' . join( ',', split /\s+/, $conf->{exportedAttr} ) . ']';
-        }
-        else {
-            my @attributes = (
-                'authenticationLevel', 'groups',
-                'ipAddr',              '_startTime',
-                '_utime',              '_lastSeen',
-                '_session_id',         '_session_kind',
+        if ( !$conf->{exportedAttr} || $conf->{exportedAttr} =~ /^\s*\+/ ) {
+            my @attributes = qw(
+              _lastSeen  _session_id  _session_kind _startTime _utime
+              authenticationLevel     groups        ipAddr
             );
             if ( my $exportedAttr = $conf->{exportedAttr} ) {
                 $exportedAttr =~ s/^\s*\+\s+//;
-                @attributes = ( @attributes, split( /\s+/, $exportedAttr ) );
-
-                # Convert @attributes into hash to remove duplicates
-                my %attributes = map( { $_ => 1 } @attributes );
-                return '[' . join( ',', keys %attributes ) . ']';
+                @attributes = ( @attributes, split /[,\s]+/, $exportedAttr );
+            }
+            else {
+                @attributes = (
+                    @attributes,
+                    keys %{ $conf->{macros} },
+                    keys %{ $conf->{exportedVars} }
+                );
             }
 
-            # Convert @attributes into hash to remove duplicates
-            my %attributes = map( { $_ => 1 } @attributes );
-            %attributes = (
-                %attributes,
-                %{ $conf->{exportedVars} },
-                %{ $conf->{macros} },
-            );
-            return '[' . join( ',', keys %attributes ) . ']';
+            # Dedup @attributes
+            %attributes = map { $_ => 1 } @attributes;
+            return '[' . join( ',', sort keys %attributes ) . ']';
+        }
+        else {
+            return
+              '['
+              . join( ',', sort split /[,\s]+/, $conf->{exportedAttr} ) . ']';
         }
     }
 );
@@ -299,7 +296,7 @@ sub newSession {
       unless ($session);
 
     $self->logger->debug(
-        "SOAP request create a new session (" . $session->id . ")" );
+        "REST request create a new session (" . $session->id . ")" );
 
     return $self->p->sendJSONresponse( $req,
         { result => 1, session => $session->data } );
@@ -314,26 +311,29 @@ sub newAuthSession {
         return $self->p->sendError( $req, 'Bad secret', 403 );
     }
 
+    $req->env->{AuthBasic} = 1;
     $req->{id}    = $id;
     $req->{force} = 1;
     $req->user( $req->param('user') );
     $req->data->{password} = $req->param('password');
     $req->steps( [
             @{ $self->p->beforeAuth },
-            qw(getUser extractFormInfo authenticate setAuthSessionInfo),
+            $self->p->authProcess,
             @{ $self->p->betweenAuthAndData },
             $self->p->sessionData,
             @{ $self->p->afterData },
             $self->p->validSession,
-            @{ $self->p->endAuth },
+            @{ $self->p->endAuth }
         ]
     );
     $req->{error} = $self->p->process($req);
     $self->logger->debug(
         "REST authentication result for $req->{user}: code $req->{error}");
 
-    return $self->p->sendError( $req, 'Bad credentials', 401 )
-      if ( $req->error > 0 );
+    if ( $req->error != 0 ) {
+        $self->p->deleteSession($req);
+        return $self->p->sendError( $req, 'Unauthorized', 401 );
+    }
     return $self->session( $req, $id );
 }
 
@@ -433,14 +433,13 @@ sub getMyKey {
     $key ||= '';
     if ($key) {
         $self->logger->debug(
-            "Request to get personal session info -> Key: $key");
+            "REST request to get personal session attribute: $key");
     }
     else {
         my $keys = $self->exportedAttr;
-        $keys =~ s/(?:\[|\])//g;
+        $keys =~ s/\[|\]//g;
         $keys =~ s/,/, /g;
-        $self->logger->debug(
-            "Request to get exported attributes -> Keys: $keys");
+        $self->logger->debug("REST request to get exported attributes: $keys");
     }
 
     return $self->session(
